@@ -1,5 +1,5 @@
 # pylint: disable-msg=W0622
-# Copyright (c) 2003-2004 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -16,21 +16,22 @@
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """pylint packaging information"""
 
-__revision__ = '$Id: __pkginfo__.py,v 1.27 2004/10/19 14:13:41 syt Exp $'
+__revision__ = '$Id: __pkginfo__.py,v 1.37 2005/04/14 10:09:31 syt Exp $'
 
 
 modname = 'pylint'
 
-numversion = (0, 5, 0)
+numversion = (0, 6, 4)
 version = '.'.join([str(num) for num in numversion])
 
 license = 'GPL'
-copyright = '''Copyright (c) 2003-2004 LOGILAB S.A. (Paris, FRANCE).
+copyright = '''Copyright (c) 2003-2005 Sylvain Thenault (thenault@gmail.com).
+Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr'''
 
 short_desc = "python code static checker"
 long_desc = """\
- Pylint is a Python source code analizer which looks for programming
+ Pylint is a Python source code analyzer which looks for programming
  errors, helps enforcing a coding standard and sniffs for some code
  smells (as defined in Martin Fowler's Refactoring book)
  .
@@ -41,15 +42,14 @@ long_desc = """\
  standard, or checking if declared interfaces are truly implemented,
  and much more.
  .
- Additionnaly, it is possible to write plugins to add your own
- checks, using a plugin mechanism."""
+ Additionally, it is possible to write plugins to add your own checks."""
 
 author = "Sylvain Thenault"
 author_email = "sylvain.thenault@logilab.fr"
 
 web = "http://www.logilab.org/projects/%s" % modname
 ftp = "ftp://ftp.logilab.org/pub/%s" % modname
-mailinglist = "mailto://python-projects@lists.logilab.org"
+mailinglist = "mailto://python-projects@logilab.org"
 
 from os.path import join
 scripts = [join('bin', filename)
@@ -59,7 +59,7 @@ subpackage_of = 'logilab'
 
 include_dirs = [join('test', 'input'), join('test', 'messages')]
 
-pyversions = ["2.2", "2.3"]
+pyversions = ["2.2", "2.3", "2.4"]
 
 debian_maintainer = 'Alexandre Fayolle'
 debian_maintainer_email = 'afayolle@debian.org'
