@@ -10,17 +10,17 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-""" Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
+""" Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
 exceptions checkers for Python code
 """
 
-__revision__ = '$Id: exceptions.py,v 1.11 2004/08/27 09:14:42 syt Exp $'
+__revision__ = '$Id: exceptions.py,v 1.17 2005/02/16 14:48:29 syt Exp $'
 
 from logilab.common import astng
 
-from logilab.pylint.checkers import BaseChecker, CheckerHandler
+from logilab.pylint.checkers import BaseChecker
 from logilab.pylint.checkers.utils import is_empty, get_names
 from logilab.pylint.interfaces import IASTNGChecker
 
@@ -38,6 +38,10 @@ MSGS = {
     'W0705': ('Bad except clauses order',
               'Used when except clauses are not in the correct order (from the \
               more specific to the more generic).'),
+    'W0706': (
+    'Identifier %s used to raise an exception is assigned to %s line %s',
+    'Used when a variable used to raise an exception is initially \
+    assigned to a value which can\'t be used as an exception.'),
     }
 
 def is_raising(stmt):
@@ -48,7 +52,7 @@ def is_raising(stmt):
             return 1
     return 0
     
-class ExceptionsChecker(BaseChecker, CheckerHandler):
+class ExceptionsChecker(BaseChecker):
     """checks for                                                              
     * excepts without exception filter                                         
     * string exceptions                                                        
@@ -61,26 +65,40 @@ class ExceptionsChecker(BaseChecker, CheckerHandler):
     priority = -4
     options = ()
 
-    def __init__(self, linter=None):
-        BaseChecker.__init__(self, linter)
-        CheckerHandler.__init__(self)
-
     def visit_raise(self, node):
         """check for string exception
         """
         # ignore empty raise
         if node.expr1 is None:
             return
-        if isinstance(node.expr1, astng.Const):
+        if isinstance(node.expr1, (astng.Const, astng.Mod)):
             self.add_message('W0701', node=node)
         else:
+            name = get_names(node.expr1)[0]
             try:
-                value = node.resolve(get_names(node.expr1)[0])
+                value = node.resolve(name)
             except astng.ResolveError:
                 pass
             else:
-                if isinstance(value, astng.Const):
-                    self.add_message('W0701', node=node)
+                if not isinstance(value, (astng.Class, astng.Module,
+                                          astng.Getattr)):
+                    if isinstance(value, astng.Const) and \
+                           value.value in (None, True, False):
+                        # this Const has been generated by resolve
+                        # since None, True and False are represented by Name
+                        # nodes in the ast, and so this const node doesn't
+                        # have the necessary parent, lineno and so on attributes
+                        frame = node.get_frame()
+                        try:
+                            lineno = frame.locals[name].source_line()
+                        except KeyError:
+                            root = frame.root()
+                            lineno = root.globals[name].source_line()
+                    else:
+                        lineno = value.source_line()
+                    self.add_message('W0706', node=node, args=(
+                        node.expr1.as_string(), value.as_string(),
+                        lineno))
             
     def visit_tryexcept(self, node):
         """check for empty except
@@ -90,9 +108,6 @@ class ExceptionsChecker(BaseChecker, CheckerHandler):
             stmt = node.handlers[index][2]
             if exc_type is None:
                 if len(node.handlers) == 1 and not is_raising(stmt):
-                    # FIXME: unable to get the correct line num !
-                    # need to patch ast
-                    
                     self.add_message('W0702', node=stmt.nodes[0])
             elif exc_type.as_string() == 'Exception':
                 # check if a "except Exception:" is followed by some other
@@ -100,8 +115,7 @@ class ExceptionsChecker(BaseChecker, CheckerHandler):
                 if index < (len(node.handlers) - 1):
                     self.add_message('W0705', node=exc_type)
                 elif index == 0 and not is_raising(stmt):
-                    self.add_message('W0703', node=exc_type)
-                    
+                    self.add_message('W0703', node=exc_type)                    
             # check for except  doing nothing but "pass", without else clause
             elif is_empty(stmt) and not node.else_:
                 self.add_message('W0704', node=exc_type)
