@@ -22,7 +22,7 @@ http://www.python.org/doc/essays/styleguide.html
 Some parts of the process_token method is based from The Tab Nanny std module.
 """
 
-__revision__ = "$Id: format.py,v 1.40 2004/09/07 06:42:40 uid1004 Exp $"
+__revision__ = "$Id: format.py,v 1.47 2005/02/24 16:58:35 syt Exp $"
 
 from os import linesep
 import re
@@ -30,10 +30,10 @@ import tokenize
 if not hasattr(tokenize, 'NL'):
     raise ValueError("tokenize.NL doesn't exist -- tokenize module too old")
 
-from logilab.common.textutils import searchall, pretty_match
+from logilab.common.textutils import pretty_match
 
 from logilab.pylint.interfaces import IRawChecker, IASTNGChecker
-from logilab.pylint.checkers import BaseRawChecker, CheckerHandler
+from logilab.pylint.checkers import BaseRawChecker
 
 MSGS = {
     'C0301': ('Line too long (%s/%s)',
@@ -73,11 +73,11 @@ should use a upper case "L" since the letter "l" looks too much like the digit \
 # simple quoted string rgx
 SQSTRING_RGX = r'"([^"\\]|\\.)*("|\\$)'
 # triple quoted string rgx
-TQSTRING_RGX = r'"""([^"\\]|\\.)*("""|$)'
+TQSTRING_RGX = r'"""([^"]|"(?!""))*("""|$)'
 # simple apostrophed rgx
 SASTRING_RGX = r"'([^'\\]|\\.)*('|\\$)"
 # triple apostrophed string rgx # FIXMY english please
-TASTRING_RGX = r"'''([^'\\]|\\.)*('''|$)"
+TASTRING_RGX = r"'''([^']|'(?!''))*('''|$)"
 
 # finally, the string regular expression
 STRING_RGX = re.compile('%s|%s' % (SQSTRING_RGX, SASTRING_RGX))
@@ -118,7 +118,7 @@ def get_string_coords(line):
     """return a list of string positions (tuple (start, end)) in the line
     """
     result = []
-    for match in searchall(STRING_RGX, line):
+    for match in re.finditer(STRING_RGX, line):
         result.append( (match.start(), match.end()) )
     return result
 
@@ -141,7 +141,7 @@ def check_line(line, writer):
     for rgx_match, rgx_search, msg_id in BAD_CONSTRUCT_RGXS:
         if rgx_match.match(clean_str):
             string_positions = get_string_coords(line)
-            for match in searchall(rgx_search, line):
+            for match in re.finditer(rgx_search, line):
                 if not in_coords(match, string_positions):
                     if line.endswith(linesep):
                         line = line[:-len(linesep)]
@@ -150,12 +150,12 @@ def check_line(line, writer):
 
             
  
-class FormatChecker(BaseRawChecker, CheckerHandler):
+class FormatChecker(BaseRawChecker):
     """checks for :                                                            
     * unauthorized constructions                                               
     * strict indentation                                                       
     * line length                                                              
-    * use of <>
+    * use of <> instead of !=
     """
     
     __implements__ = (IRawChecker, IASTNGChecker)
@@ -180,7 +180,6 @@ class FormatChecker(BaseRawChecker, CheckerHandler):
                )
     def __init__(self, linter=None):
         BaseRawChecker.__init__(self, linter)
-        CheckerHandler.__init__(self)
         self._lines = None
         self._visited_lines = None
 
@@ -261,26 +260,12 @@ class FormatChecker(BaseRawChecker, CheckerHandler):
             return
         prev_sibl = node.previous_sibling()
         if prev_sibl is not None:
+            # don't use .source_line since it cause C0321 false positive !
             prev_line = prev_sibl.lineno
         else:
+            # itou ?
             prev_line = node.parent.get_statement().lineno
-        line = node.lineno
-        if line is None:
-            # FIXME: log node with the missing line no
-            _node = node
-            try:
-                while line is None:
-                    _node = _node.getChildNodes()[0]
-                    line = _node.lineno
-            except IndexError:
-                _node = node.parent
-                while _node and line is None:
-                    line = _node.lineno
-                    _node = _node.parent
-                
-        if line is None:
-            # FIXME: log node with the missing line no
-            return
+        line = node.source_line()
         if prev_line == line and self._visited_lines.get(line) != 2:
             self.add_message('C0321', node=node)
             self._visited_lines[line] = 2
@@ -288,9 +273,15 @@ class FormatChecker(BaseRawChecker, CheckerHandler):
         if self._visited_lines.has_key(line):
             return
         self._visited_lines[line] = 1
-        msg_def = check_line(self._lines[line], self)
-        if msg_def:
-            self.add_message(msg_def[0], node = node, args=msg_def[1])
+        #print 'checking line', self._lines[line]
+        #print node
+        try:
+            msg_def = check_line(self._lines[line], self)
+            if msg_def:
+                self.add_message(msg_def[0], node = node, args=msg_def[1])
+        except KeyError:
+            # FIXME: internal error !
+            pass
 
     def check_lines(self, lines, start):
         """check lines againt bad construction regexp
@@ -308,6 +299,8 @@ class FormatChecker(BaseRawChecker, CheckerHandler):
         """return the indent level of the string
         """
         indent = self.config.indent_string
+        if indent == '\\t': # \t is not interpreted in the configuration file
+            indent = '\t'
         level = 0
         unit_size = len(indent)
         while string[:unit_size] == indent:
