@@ -10,22 +10,22 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-""" Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
+""" Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
 variables checkers for Python code
 """
 
-__revision__ = "$Id: variables.py,v 1.36 2004/08/25 14:29:05 syt Exp $"
+__revision__ = "$Id: variables.py,v 1.45 2005/04/14 09:32:09 syt Exp $"
 
 from copy import copy
 
 from logilab.common import astng
 
 from logilab.pylint.interfaces import IASTNGChecker
-from logilab.pylint.checkers import BaseChecker, CheckerHandler
-from logilab.pylint.checkers.utils import is_interface, is_abstract, \
-     is_builtin, is_native_builtin, is_error
+from logilab.pylint.checkers import BaseChecker
+from logilab.pylint.checkers.utils import is_interface, is_error, is_builtin, \
+     is_native_builtin, is_abstract, are_exclusive, get_nodes_from_class
 
     
 MSGS = {
@@ -51,7 +51,7 @@ MSGS = {
               'Used when a variable or function override a built-in.'),
     }
 
-class VariablesChecker(BaseChecker, CheckerHandler):
+class VariablesChecker(BaseChecker):
     """checks for                                                              
     * unused variables / imports                                               
     * undefined variables                                                      
@@ -77,7 +77,6 @@ __init__ files.'}),
                )
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
-        CheckerHandler.__init__(self)
         self._to_consume = None
         
     def visit_module(self, node):
@@ -99,9 +98,12 @@ __init__ files.'}),
             return
         for name, stmt in not_consumed.items():
             # the latest test avoid warning on __builtins__ which is 
-            # implicitly added by wildcard import 
-            if name == '__builtins__':
-                continue
+            # implicitly added by wildcard import
+            # update 20 janv 2005: no more necessary since we are no more
+            # exec'ing "from module import *" and so __builtins__ is no more
+            # added to the global scope
+            #if name == '__builtins__':
+            #    continue
             if isinstance(stmt, astng.Import) or (
                 isinstance(stmt, astng.From) and stmt.modname != '__future__'):
                 self.add_message('W0611', args=name, node=stmt)
@@ -112,7 +114,7 @@ __init__ files.'}),
         """
         self._to_consume.append((copy(node.locals), {}, 'class'))
             
-    def leave_class(self, node):
+    def leave_class(self, _):
         """leave class: update consumption analysis variable
         """
         # do not check for not used locals here (no sense)
@@ -123,7 +125,7 @@ __init__ files.'}),
         """
         self._to_consume.append((copy(node.locals), {}, 'class'))
             
-    def leave_lambda(self, node):
+    def leave_lambda(self, _):
         """leave lambda: update consumption analysis variable
         """
         # do not check for not used locals here
@@ -187,21 +189,18 @@ __init__ files.'}),
         redefine a built-in
         """
         name = node.name
-        stmt = node
+        stmt = node.get_statement()
         frame = stmt.get_frame()
         # if the name node is used as a function default argument's value, then
         # start from the parent frame of the function instead of the function
         # frame
-        if is_func_default(node):
+        if is_func_default(node) or is_ancestor_name(frame, node):
             start_index = len(self._to_consume) - 2
         else:
             start_index = len(self._to_consume) - 1
         # iterates through the parent scope, from the inner to the outer
         for i in range(start_index, -1, -1):
             to_consume, consumed, scope_type = self._to_consume[i]
-            # the name has already been consumed, ends
-            if consumed.has_key(name):
-                break
             # if the current scope is a class scope but it's not the inner scope
             # ignore it
             #
@@ -209,15 +208,22 @@ __init__ files.'}),
             # function members when there are some common names
             if scope_type == 'class' and i != start_index:
                 continue
+            # the name has already been consumed, ends
+            if consumed.has_key(name):
+                break
             # mark the name as consumed if it's defined in this scope
             # (ie no KeyError is raise by "to_consume[name]"
             try:
                 consumed[name] = def_stmt = to_consume[name].get_statement()
                 del to_consume[name]
                 # checks for use before assigment
+                # FIXME: the last condition should just check attribute access
+                # is protected by a try: except NameError:
                 if def_stmt and frame is consumed[name].get_frame() and \
-                       stmt.lineno < def_stmt.lineno:
-                    self.add_message('E0601', args=name, node=stmt)
+                       stmt.source_line() <= def_stmt.source_line() and \
+                       not list_comp_var(node) and \
+                       not are_exclusive(stmt, def_stmt):
+                    self.add_message('E0601', args=name, node=node)
                 break
             except KeyError:
                 continue
@@ -228,6 +234,26 @@ __init__ files.'}),
                 self._to_consume[-1][1][name] = 1
                 self.add_message('E0602', args=name, node=stmt)
 
+try:
+    # python >= 2.4
+    COMP_NODE_TYPES = (astng.ListComp, astng.GenExpr)
+except AttributeError:
+    COMP_NODE_TYPES = astng.ListComp
+    
+def list_comp_var(var_node, comp_node_types=COMP_NODE_TYPES):
+    """return True if the variable node is defined by a parent list
+    comprehension
+    """
+    _node = var_node.parent
+    # FIXME: to finish !!!
+    while _node:
+        if isinstance(_node, comp_node_types):
+            for ass_node in get_nodes_from_class(_node, astng.AssName):
+                if ass_node.name == var_node.name:
+                    return True
+        _node = _node.parent
+    return False
+
 def is_func_default(node):
     """return true if the name is used in function default argument's value
     """
@@ -239,7 +265,19 @@ def is_func_default(node):
         return 1
     return is_func_default(parent)
     
-    
+def is_ancestor_name(frame, node):
+    """return True if `frame` is a astng.Class node with `node` in the
+    subtree of its bases attribute
+    """
+    try:
+        bases = frame.bases
+    except AttributeError:
+        return False
+    for base in bases:
+        if node in get_nodes_from_class(base, astng.Name):
+            return True
+    return False
+
 def register(linter):
     """required method to auto register this checker"""
     linter.register_checker(VariablesChecker(linter))
