@@ -50,3 +50,9 @@ def other(funky):
     """yo, test formatted result with indentation"""
     funky= funky+2
     
+html = """<option value="=">ist genau gleich</option>
+yo+=4
+"""
+html2 = """<option value='='>ist genau gleich</option>
+yo+=4
+"""
