@@ -14,14 +14,14 @@
  http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
-__revision__ = "$Id: smoketest.py,v 1.4 2004/04/24 14:20:25 syt Exp $"
+__revision__ = "$Id: smoketest.py,v 1.5 2005/01/20 15:12:56 syt Exp $"
 
 import unittest
 import sys
 from cStringIO import StringIO
 
 from logilab.pylint.lint import Run
-from logilab.pylint.reporters.text import TextReporter, TextReporter2
+from logilab.pylint.reporters.text import TextReporter, TextReporter2, ColorizedTextReporter
 from logilab.pylint.reporters.html import HTMLReporter
 
     
@@ -29,7 +29,7 @@ class LintSmokeTest(unittest.TestCase):
         
     def test1(self):
         """make pylint checking itself"""
-        Run(['logilab.pylint'], reporter=TextReporter(StringIO()), quiet=1)
+        Run(['--include-ids=y', 'logilab.pylint'], reporter=TextReporter(StringIO()), quiet=1)
     
     def test2(self):
         """make pylint checking itself"""
@@ -39,6 +39,10 @@ class LintSmokeTest(unittest.TestCase):
         """make pylint checking itself"""
         Run(['logilab.pylint.checkers'], reporter=HTMLReporter(StringIO()), quiet=1)
     
+    def test4(self):
+        """make pylint checking itself"""
+        Run(['logilab.pylint.checkers'], reporter=ColorizedTextReporter(StringIO()), quiet=1)
+    
     def test_generate_config_option(self):
         """make pylint checking itself"""
         sys.stdout = StringIO()
