@@ -16,7 +16,7 @@
 Check format checker helper functions
 """
 
-__revision__ = '$Id: test_format.py,v 1.10 2004/10/19 13:41:01 syt Exp $'
+__revision__ = '$Id: test_format.py,v 1.11 2005/01/20 15:12:56 syt Exp $'
 
 import unittest
 import sys
@@ -93,6 +93,10 @@ class ChecklineFunctionTest(unittest.TestCase):
         
     def test_known_values_opspace_3(self):
         self.assertEqual(ulines(check_line('a =1', REPORTER)), ('C0323', 'a =1\n  ^'))
+
+    def test_known_values_opspace_4(self):
+        self.assertEqual(check_line('f(a=1)', REPORTER), None)
+
     def test_known_values_opspace_4(self):
         self.assertEqual(check_line('f(a=1)', REPORTER), None)
 
@@ -147,6 +151,11 @@ class ChecklineFunctionTest(unittest.TestCase):
         self.assertEqual(ulines(check_line("self.filterFunc = eval('lambda %s: %s'%(','.join(variables),formula),{},{})", REPORTER)),
                          ('C0324', "self.filterFunc = eval('lambda %s: %s'%(','.join(variables),formula),{},{})\n                                                           ^^"))
 
+    def test_known_values_tqstring(self):
+        self.assertEqual(check_line('print """<a="=")', REPORTER), None)
+        
+    def test_known_values_tastring(self):
+        self.assertEqual(check_line("print '''<a='=')", REPORTER), None)
         
 if __name__ == '__main__':
     unittest.main()
